#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Copyright (c) 2018, 2019 Kairo de Araujo
"""

__title__ = "ownca"
__description__ = "Python Own Certificate Authority"
__url__ = "https://github.com/kairoaraujo/ownca"
__version__ = "0.0.1-alpha4"
__author__ = "Kairo de Araujo"
__author_email__ = "kairo@dearaujo.nl"
__license__ = "Apache 2.0"
__copyright__ = "Copyright 2019 Kairo de Araujo"
